import { existsSync, lstatSync, readFileSync } from 'node:fs';
import { delimiter, dirname, join, resolve, sep } from 'node:path';
import { fileURLToPath } from 'node:url';
import process from 'node:process';
import arrify from 'arrify';
import { uniq, compact } from 'lodash-es';
import { globbySync } from 'globby';
import slash from 'slash';
import createdLogger from 'debug';
import { execaOutput } from './util/util.js';
const __filename = fileURLToPath(import.meta.url);
const __dirname = dirname(__filename);
// eslint-disable-next-line @typescript-eslint/naming-convention
const PROJECT_ROOT = join(__dirname, '..');
// eslint-disable-next-line @typescript-eslint/naming-convention
const PACKAGE_NAME_PATTERN = [JSON.parse(readFileSync(join(PROJECT_ROOT, 'package.json')).toString()).name];
const win32 = process.platform === 'win32';
const nvm = process.env.NVM_HOME;
const debug = createdLogger('yeoman:environment');
/**
 * Search for npm packages.
 */
export function moduleLookupSync(options, find) {
    debug('Running lookup with options: %o', options);
    options = { ...options };
    options.filePatterns = arrify(options.filePatterns ?? 'package.json').map(filePattern => slash(filePattern));
    if (options.packagePaths) {
        options.packagePaths = arrify(options.packagePaths);
        if (options.reverse) {
            options.packagePaths = options.packagePaths.reverse();
        }
    }
    else {
        options.npmPaths = options.npmPaths ?? getNpmPaths(options);
        if (options.reverse && Array.isArray(options.npmPaths)) {
            options.npmPaths = options.npmPaths.reverse();
        }
        options.packagePatterns = arrify(options.packagePatterns ?? PACKAGE_NAME_PATTERN).map(packagePattern => slash(packagePattern));
        options.packagePaths = findPackagesIn(options.npmPaths, options.packagePatterns);
    }
    debug('Lookup calculated options: %o', options);
    const modules = [];
    for (const packagePath of options.packagePaths) {
        if (!existsSync(packagePath) || (!lstatSync(packagePath).isDirectory() && !lstatSync(packagePath).isSymbolicLink())) {
            continue;
        }
        const files = globbySync(options.filePatterns, {
            cwd: packagePath,
            absolute: true,
            ...options.globbyOptions,
        });
        const filePath = find({ files, packagePath });
        if (filePath) {
            return [{ filePath, packagePath }];
        }
        for (const filePath of files) {
            modules.push({ filePath, packagePath });
        }
    }
    return modules;
}
/**
 * Search npm for every available generators.
 * Generators are npm packages who's name start with `generator-` and who're placed in the
 * top level `node_module` path. They can be installed globally or locally.
 *
 * @method
 *
 * @param searchPaths List of search paths
 * @param packagePatterns Pattern of the packages
 * @param globbyOptions
 * @return List of the generator modules path
 */
export function findPackagesIn(searchPaths, packagePatterns, globbyOptions) {
    searchPaths = arrify(searchPaths)
        .filter(Boolean)
        .map(npmPath => resolve(npmPath));
    let modules = [];
    for (const root of searchPaths) {
        if (!existsSync(root) || (!lstatSync(root).isDirectory() && !lstatSync(root).isSymbolicLink())) {
            continue;
        }
        // Some folders might not be readable to the current user. For those, we add a try
        // catch to handle the error gracefully as globby doesn't have an option to skip
        // restricted folders.
        try {
            modules = modules.concat(globbySync(packagePatterns, {
                cwd: root,
                onlyDirectories: true,
                expandDirectories: false,
                absolute: true,
                deep: 0,
                ...globbyOptions,
            }));
            // To limit recursive lookups into non-namespace folders within globby,
            // fetch all namespaces in root, then search each namespace separately
            // for generator modules
            const scopes = globbySync(['@*'], {
                cwd: root,
                onlyDirectories: true,
                expandDirectories: false,
                absolute: true,
                deep: 0,
                ...globbyOptions,
            });
            for (const scope of scopes) {
                modules = modules.concat(globbySync(packagePatterns, {
                    cwd: scope,
                    onlyDirectories: true,
                    expandDirectories: false,
                    absolute: true,
                    deep: 0,
                    ...globbyOptions,
                }));
            }
        }
        catch (error) {
            debug('Could not access %s (%s)', root, error);
        }
    }
    return modules;
}
/**
 * Get the npm lookup directories (`node_modules/`)
 *
 * @method
 *
 * @param {boolean|Object} [options]
 * @param {boolean} [options.localOnly = false] - Set true to skip lookups of
 *                                               globally-installed generators.
 * @param {boolean} [options.filterPaths = false] - Remove paths that don't ends
 *                       with a supported path (don't touch at NODE_PATH paths).
 * @return {Array} lookup paths
 */
export function getNpmPaths(options = {}) {
    // Resolve signature where options is boolean (localOnly).
    if (typeof options === 'boolean') {
        options = { localOnly: options };
    }
    // Start with the local paths.
    let paths = getLocalNpmPaths();
    // Append global paths, unless they should be excluded.
    if (!options.localOnly) {
        paths = paths.concat(getGlobalNpmPaths(options.filterPaths));
    }
    return uniq(paths);
}
/**
 * Get the local npm lookup directories
 * @private
 * @return {Array} lookup paths
 */
function getLocalNpmPaths() {
    const paths = [];
    // Walk up the CWD and add `node_modules/` folder lookup on each level
    process
        .cwd()
        .split(sep)
        .forEach((part, i, parts) => {
        let lookup = join(...parts.slice(0, i + 1), 'node_modules');
        if (!win32) {
            lookup = `/${lookup}`;
        }
        paths.push(lookup);
    });
    return uniq(paths.reverse());
}
/**
 * Get the global npm lookup directories
 * Reference: https://nodejs.org/api/modules.html
 * @private
 * @return {Array} lookup paths
 */
function getGlobalNpmPaths(filterPaths = true) {
    let paths = [];
    // Node.js will search in the following list of GLOBAL_FOLDERS:
    // 1: $HOME/.node_modules
    // 2: $HOME/.node_libraries
    // 3: $PREFIX/lib/node
    const filterValidNpmPath = function (path, ignore = false) {
        return ignore ? [path] : ['/node_modules', '/.node_modules', '/.node_libraries', '/node'].some(dir => path.endsWith(dir)) ? [path] : [];
    };
    // Default paths for each system
    if (nvm && process.env.NVM_HOME) {
        paths.push(join(process.env.NVM_HOME, process.version, 'node_modules'));
    }
    else if (win32 && process.env.APPDATA) {
        paths.push(join(process.env.APPDATA, 'npm/node_modules'));
    }
    else {
        paths.push('/usr/lib/node_modules', '/usr/local/lib/node_modules');
    }
    // Add NVM prefix directory
    if (process.env.NVM_PATH) {
        paths.push(join(dirname(process.env.NVM_PATH), 'node_modules'));
    }
    // Adding global npm directories
    // We tried using npm to get the global modules path, but it haven't work out
    // because of bugs in the parseable implementation of `ls` command and mostly
    // performance issues. So, we go with our best bet for now.
    if (process.env.NODE_PATH) {
        paths = compact(process.env.NODE_PATH.split(delimiter)).concat(paths);
    }
    // Global node_modules should be 4 or 2 directory up this one (most of the time)
    // Ex: /usr/another_global/node_modules/yeoman-denerator/node_modules/yeoman-environment/lib (1 level dependency)
    paths.push(...filterValidNpmPath(join(PROJECT_ROOT, '../../..'), !filterPaths));
    // Ex: /usr/another_global/node_modules/yeoman-environment/lib (installed directly)
    paths.push(join(PROJECT_ROOT, '..'));
    // Get yarn global directory and infer the module paths from there
    const yarnBase = execaOutput('yarn', ['global', 'dir'], { encoding: 'utf8' });
    if (yarnBase) {
        paths.push(resolve(yarnBase, 'node_modules'));
        paths.push(resolve(yarnBase, '../link/'));
    }
    // Get npm global prefix and infer the module paths from there
    const globalInstall = execaOutput('npm', ['root', '-g'], {
        encoding: 'utf8',
    });
    if (globalInstall) {
        paths.push(resolve(globalInstall));
    }
    // Adds support for generator resolving when yeoman-generator has been linked
    if (process.argv[1]) {
        paths.push(...filterValidNpmPath(join(dirname(process.argv[1]), '../..'), !filterPaths));
    }
    return uniq(paths.filter(Boolean).reverse());
}
